@@ -1,27 +1,7 @@
-/* PipeWire
- *
- * Copyright © 2020 Georges Basile Stavracas Neto
- * Copyright © 2021 Wim Taymans <wim.taymans@gmail.com>
- *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice (including the next
- * paragraph) shall be included in all copies or substantial portions of the
- * Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
- */
+/* PipeWire */
+/* SPDX-FileCopyrightText: Copyright © 2020 Georges Basile Stavracas Neto */
+/* SPDX-FileCopyrightText: Copyright © 2021 Wim Taymans <wim.taymans@gmail.com> */
+/* SPDX-License-Identifier: MIT */
 
 #include <stdlib.h>
 #include <string.h>
@@ -31,6 +11,7 @@
 #include <spa/utils/list.h>
 #include <spa/utils/hook.h>
 #include <spa/utils/string.h>
+#include <pipewire/cleanup.h>
 #include <pipewire/log.h>
 #include <pipewire/map.h>
 #include <pipewire/properties.h>
@@ -137,7 +118,8 @@ int module_unload(struct module *module)
 /** utils */
 void module_args_add_props(struct pw_properties *props, const char *str)
 {
-	char *s = strdup(str), *p = s, *e, f;
+	spa_autofree char *s = strdup(str);
+	char *p = s, *e, f;
 	const char *k, *v;
 	const struct str_map *map;
 
@@ -161,14 +143,14 @@ void module_args_add_props(struct pw_properties *props, const char *str)
 			f = ' ';
 		}
 		v = p;
-		for (e = p; *e ; e++) {
-			if (*e == f)
+		for (e = p; *p ;) {
+			if (*p == f)
 				break;
-			if (*e == '\\')
-				e++;
+			if (*p == '\\')
+				p++;
+			*e++ = *p++;
 		}
-		p = e;
-		if (*e != '\0')
+		if (*p != '\0')
 			p++;
 		*e = '\0';
 
@@ -180,44 +162,72 @@ void module_args_add_props(struct pw_properties *props, const char *str)
 		}
 		pw_properties_set(props, k, v);
 	}
-	free(s);
 }
 
-int module_args_to_audioinfo(struct impl *impl, struct pw_properties *props, struct spa_audio_info_raw *info)
+static bool find_key(const char * const keys[], const char *key)
+{
+	for (int i = 0; keys[i] != NULL; i++)
+		if (spa_streq(keys[i], key))
+			return true;
+	return false;
+}
+
+static int module_args_check(struct pw_properties *props, const char * const valid_args[])
+{
+	if (valid_args != NULL) {
+		const struct spa_dict_item *it;
+		spa_dict_for_each(it, &props->dict) {
+			if (!find_key(valid_args, it->key)) {
+				pw_log_warn("'%s' is not a valid module argument key", it->key);
+				return -EINVAL;
+			}
+		}
+	}
+	return 0;
+}
+
+int module_args_to_audioinfo_keys(struct impl *impl, struct pw_properties *props,
+		const char *key_format, const char *key_rate,
+		const char *key_channels, const char *key_channel_map,
+		struct spa_audio_info_raw *info)
 {
 	const char *str;
 	uint32_t i;
 
-	/* We don't use any incoming format setting and use our native format */
-	spa_zero(*info);
-	info->flags = SPA_AUDIO_FLAG_UNPOSITIONED;
-	info->format = SPA_AUDIO_FORMAT_F32P;
-
-	if ((str = pw_properties_get(props, "channels")) != NULL) {
-		info->channels = pw_properties_parse_int(str);
-		if (info->channels == 0 || info->channels > SPA_AUDIO_MAX_CHANNELS) {
-			pw_log_error("invalid channels '%s'", str);
+	if (key_format && (str = pw_properties_get(props, key_format)) != NULL) {
+		info->format = format_paname2id(str, strlen(str));
+		if (info->format == SPA_AUDIO_FORMAT_UNKNOWN) {
+			pw_log_error("invalid %s '%s'", key_format, str);
 			return -EINVAL;
 		}
-		pw_properties_set(props, "channels", NULL);
+		pw_properties_set(props, key_format, NULL);
 	}
-	if ((str = pw_properties_get(props, "channel_map")) != NULL) {
+	if (key_channels && (str = pw_properties_get(props, key_channels)) != NULL) {
+		info->channels = pw_properties_parse_int(str);
+		if (info->channels == 0 || info->channels > SPA_AUDIO_MAX_CHANNELS) {
+			pw_log_error("invalid %s '%s'", key_channels, str);
+			return -EINVAL;
+		}
+		pw_properties_set(props, key_channels, NULL);
+	}
+	if (key_channel_map && (str = pw_properties_get(props, key_channel_map)) != NULL) {
 		struct channel_map map;
 
 		channel_map_parse(str, &map);
 		if (map.channels == 0 || map.channels > SPA_AUDIO_MAX_CHANNELS) {
-			pw_log_error("invalid channel_map '%s'", str);
+			pw_log_error("invalid %s '%s'", key_channel_map, str);
 			return -EINVAL;
 		}
 		if (info->channels == 0)
 			info->channels = map.channels;
 		if (info->channels != map.channels) {
-			pw_log_error("Mismatched channel map");
+			pw_log_error("Mismatched %s and %s (%d vs %d)",
+					key_channels, key_channel_map,
+					info->channels, map.channels);
 			return -EINVAL;
 		}
 		channel_map_to_positions(&map, info->position);
-		info->flags &= ~SPA_AUDIO_FLAG_UNPOSITIONED;
-		pw_properties_set(props, "channel_map", NULL);
+		pw_properties_set(props, key_channel_map, NULL);
 	} else {
 		if (info->channels == 0)
 			info->channels = impl->defs.sample_spec.channels;
@@ -234,19 +244,25 @@ int module_args_to_audioinfo(struct impl *impl, struct pw_properties *props, str
 			for (i = 0; i < info->channels; i++)
 				info->position[i] = SPA_AUDIO_CHANNEL_UNKNOWN;
 		}
-		if (info->position[0] != SPA_AUDIO_CHANNEL_UNKNOWN)
-			info->flags &= ~SPA_AUDIO_FLAG_UNPOSITIONED;
+		if (info->position[0] == SPA_AUDIO_CHANNEL_UNKNOWN)
+			info->flags |= SPA_AUDIO_FLAG_UNPOSITIONED;
 	}
-
-	if ((str = pw_properties_get(props, "rate")) != NULL) {
+	if (key_rate && (str = pw_properties_get(props, key_rate)) != NULL) {
 		info->rate = pw_properties_parse_int(str);
-		pw_properties_set(props, "rate", NULL);
-	} else {
-		info->rate = 0;
+		pw_properties_set(props, key_rate, NULL);
 	}
 	return 0;
 }
 
+int module_args_to_audioinfo(struct impl *impl, struct pw_properties *props, struct spa_audio_info_raw *info)
+{
+	/* We don't use any incoming format setting and use our native format */
+	spa_zero(*info);
+	info->format = SPA_AUDIO_FORMAT_F32P;
+	return module_args_to_audioinfo_keys(impl, props,
+			NULL, "rate", "channels", "channel_map", info);
+}
+
 bool module_args_parse_bool(const char *v)
 {
 	if (spa_streq(v, "1") || !strcasecmp(v, "y") || !strcasecmp(v, "t") ||
@@ -255,6 +271,28 @@ bool module_args_parse_bool(const char *v)
 	return false;
 }
 
+void audioinfo_to_properties(struct spa_audio_info_raw *info, struct pw_properties *props)
+{
+	uint32_t i;
+
+	if (info->format)
+		pw_properties_setf(props, SPA_KEY_AUDIO_FORMAT, "%s",
+					format_id2name(info->format));
+	if (info->rate)
+		pw_properties_setf(props, SPA_KEY_AUDIO_RATE, "%u", info->rate);
+	if (info->channels) {
+		char *s, *p;
+
+		pw_properties_setf(props, SPA_KEY_AUDIO_CHANNELS, "%u", info->channels);
+
+		p = s = alloca(info->channels * 8);
+		for (i = 0; i < info->channels; i++)
+			p += spa_scnprintf(p, 8, "%s%s", i == 0 ? "" : ", ",
+					channel_id2name(info->position[i]));
+		pw_properties_setf(props, SPA_KEY_AUDIO_POSITION, "[ %s ]", s);
+	}
+}
+
 static const struct module_info *find_module_info(const char *name)
 {
 	extern const struct module_info __start_pw_mod_pulse_modules[];
@@ -283,6 +321,7 @@ struct module *module_create(struct impl *impl, const char *name, const char *ar
 {
 	const struct module_info *info;
 	struct module *module;
+	int res;
 
 	info = find_module_info(name);
 	if (info == NULL) {
@@ -305,19 +344,19 @@ struct module *module_create(struct impl *impl, const char *name, const char *ar
 		return NULL;
 
 	module->props = pw_properties_new(NULL, NULL);
-	if (module->props == NULL) {
-		module_free(module);
-		return NULL;
-	}
+	if (module->props == NULL)
+		goto error_free;
 
 	if (args)
 		module_args_add_props(module->props, args);
-
-	int res = module->info->prepare(module);
-	if (res < 0) {
-		module_free(module);
+	if ((res = module_args_check(module->props, info->valid_args)) < 0) {
 		errno = -res;
-		return NULL;
+		goto error_free;
+	}
+
+	if ((res = module->info->prepare(module)) < 0) {
+		errno = -res;
+		goto error_free;
 	}
 
 	module->index = pw_map_insert_new(&impl->modules, module);
@@ -330,4 +369,9 @@ struct module *module_create(struct impl *impl, const char *name, const char *ar
 	module->index |= MODULE_FLAG;
 
 	return module;
+
+error_free:
+	module_free(module);
+	return NULL;
+
 }
